package org.apache.bcel.generic;

import org.apache.bcel.util.ByteSequence;
import java.io.IOException;
import java.io.DataOutputStream;

public abstract class Select extends BranchInstruction implements VariableLengthInstruction, StackConsumer, StackProducer
{
    @Deprecated
    protected int[] match;
    @Deprecated
    protected int[] indices;
    @Deprecated
    protected InstructionHandle[] targets;
    @Deprecated
    protected int fixed_length;
    @Deprecated
    protected int match_length;
    @Deprecated
    protected int padding;
    
    Select() {
    }
    
    Select(final short opcode, final int[] match, final InstructionHandle[] targets, final InstructionHandle defaultTarget) {
        super(opcode, null);
        this.match = match;
        this.targets = targets;
        this.setTarget(defaultTarget);
        for (final InstructionHandle target2 : targets) {
            BranchInstruction.notifyTarget(null, target2, this);
        }
        if ((this.match_length = match.length) != targets.length) {
            throw new ClassGenException("Match and target array have not the same length: Match length: " + match.length + " Target length: " + targets.length);
        }
        this.indices = new int[this.match_length];
    }
    
    @Override
    protected Object clone() throws CloneNotSupportedException {
        final Select copy = (Select)super.clone();
        copy.match = this.match.clone();
        copy.indices = this.indices.clone();
        copy.targets = this.targets.clone();
        return copy;
    }
    
    @Override
    public boolean containsTarget(final InstructionHandle ih) {
        if (super.getTarget() == ih) {
            return true;
        }
        for (final InstructionHandle target2 : this.targets) {
            if (target2 == ih) {
                return true;
            }
        }
        return false;
    }
    
    @Override
    void dispose() {
        super.dispose();
        for (final InstructionHandle target2 : this.targets) {
            target2.removeTargeter(this);
        }
    }
    
    @Override
    public void dump(final DataOutputStream out) throws IOException {
        out.writeByte(super.getOpcode());
        for (int i = 0; i < this.padding; ++i) {
            out.writeByte(0);
        }
        super.setIndex(this.getTargetOffset());
        out.writeInt(super.getIndex());
    }
    
    final int getFixedLength() {
        return this.fixed_length;
    }
    
    public int[] getIndices() {
        return this.indices;
    }
    
    final int getIndices(final int index) {
        return this.indices[index];
    }
    
    final int getMatch(final int index) {
        return this.match[index];
    }
    
    final int getMatchLength() {
        return this.match_length;
    }
    
    public int[] getMatchs() {
        return this.match;
    }
    
    final int getPadding() {
        return this.padding;
    }
    
    final InstructionHandle getTarget(final int index) {
        return this.targets[index];
    }
    
    public InstructionHandle[] getTargets() {
        return this.targets;
    }
    
    @Override
    protected void initFromFile(final ByteSequence bytes, final boolean wide) throws IOException {
        this.padding = (4 - bytes.getIndex() % 4) % 4;
        for (int i = 0; i < this.padding; ++i) {
            bytes.readByte();
        }
        super.setIndex(bytes.readInt());
    }
    
    final void setFixedLength(final int fixedLength) {
        this.fixed_length = fixedLength;
    }
    
    final int setIndices(final int i, final int value) {
        return this.indices[i] = value;
    }
    
    final void setIndices(final int[] array) {
        this.indices = array;
    }
    
    final void setMatch(final int index, final int value) {
        this.match[index] = value;
    }
    
    final int setMatchLength(final int matchLength) {
        return this.match_length = matchLength;
    }
    
    final void setMatches(final int[] array) {
        this.match = array;
    }
    
    public void setTarget(final int i, final InstructionHandle target) {
        BranchInstruction.notifyTarget(this.targets[i], target, this);
        this.targets[i] = target;
    }
    
    final void setTargets(final InstructionHandle[] array) {
        this.targets = array;
    }
    
    @Override
    public String toString(final boolean verbose) {
        final StringBuilder buf = new StringBuilder(super.toString(verbose));
        if (verbose) {
            for (int i = 0; i < this.match_length; ++i) {
                String s = "null";
                if (this.targets[i] != null) {
                    s = this.targets[i].getInstruction().toString();
                }
                buf.append("(").append(this.match[i]).append(", ").append(s).append(" = {").append(this.indices[i]).append("})");
            }
        }
        else {
            buf.append(" ...");
        }
        return buf.toString();
    }
    
    @Override
    protected int updatePosition(final int offset, final int maxOffset) {
        this.setPosition(this.getPosition() + offset);
        final short oldLength = (short)super.getLength();
        this.padding = (4 - (this.getPosition() + 1) % 4) % 4;
        super.setLength((short)(this.fixed_length + this.padding));
        return super.getLength() - oldLength;
    }
    
    @Override
    public void updateTarget(final InstructionHandle oldIh, final InstructionHandle newIh) {
        boolean targeted = false;
        if (super.getTarget() == oldIh) {
            targeted = true;
            this.setTarget(newIh);
        }
        for (int i = 0; i < this.targets.length; ++i) {
            if (this.targets[i] == oldIh) {
                targeted = true;
                this.setTarget(i, newIh);
            }
        }
        if (!targeted) {
            throw new ClassGenException("Not targeting " + oldIh);
        }
    }
}
